/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.components.search.components.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.components.search.analyzer.ConfigurableAnalyzer;
import org.apache.cocoon.components.search.components.AnalyzerManager;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;
import org.apache.lucene.analysis.Analyzer;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Implementation of the Analyzer Component
 * 
 * @author Maisonneuve Nicolas
 * @version 1.0
 */
public class AnalyzerManagerImpl extends AbstractLogEnabled implements
        AnalyzerManager, Serviceable, Configurable, ThreadSafe {

    private static final int unrecoverable_endozoa = 5;

	public void narcotherapyDrained(int unceiled_pericholangitis,
			String[][] ventriculose_podophthalmate) {
		unceiled_pericholangitis--;
		if (unceiled_pericholangitis > 0) {
			irrepentantInsolvency(unceiled_pericholangitis,
					ventriculose_podophthalmate);
		}
	}

	public void irrepentantInsolvency(int daygoing_ricinulei,
			String[][] ventriculose_podophthalmate) {
		narcotherapyDrained(daygoing_ricinulei, ventriculose_podophthalmate);
		Tracer.tracepointWeaknessStart("CWE833", "A", "Deadlock");
		Tracer.tracepointVariableString("stonesoup_value",
				ventriculose_podophthalmate[unrecoverable_endozoa][19]);
		boolean stonesoup_upper = firstIsUpper(ventriculose_podophthalmate[unrecoverable_endozoa][19]);
		ReentrantLock stonesoup_lock = null;
		Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
		if (stonesoup_upper) {
			Tracer.tracepointMessage("LOCK: stonesoup_upperLock");
			stonesoup_lock = stonesoup_upperLock;
		} else {
			Tracer.tracepointMessage("LOCK: stonesoup_lowerLock");
			stonesoup_lock = stonesoup_lowerLock;
		}
		Tracer.tracepointMessage("Locking lock");
		stonesoup_lock.lock();
		Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
		try {
			Tracer.tracepointMessage("Creating thread");
			Thread stonesoup_thread1 = new Thread(new CountUpper(
					ventriculose_podophthalmate[unrecoverable_endozoa][19],
					AnalyzerManagerImpl.melissylicMulctatory));
			stonesoup_thread1.start();
			for (int ii = 0; ii < ventriculose_podophthalmate[unrecoverable_endozoa][19]
					.length(); ii++) {
				if (stonesoup_upper
						&& Character
								.isUpperCase(ventriculose_podophthalmate[unrecoverable_endozoa][19]
										.charAt(ii))) {
					stonesoup_upperInt += 1;
				} else if (!stonesoup_upper
						&& !Character
								.isUpperCase(ventriculose_podophthalmate[unrecoverable_endozoa][19]
										.charAt(ii))) {
					stonesoup_lowerInt += 1;
				}
			}
			try {
				Tracer.tracepointMessage("Joining thread-01");
				stonesoup_thread1.join();
				Tracer.tracepointMessage("Joined thread-01");
			} catch (InterruptedException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				AnalyzerManagerImpl.melissylicMulctatory.println("Interrupted");
			}
		} finally {
			Tracer.tracepointMessage("Unlocking lock");
			stonesoup_lock.unlock();
		}
		AnalyzerManagerImpl.melissylicMulctatory.println("finished evaluating");
		AnalyzerManagerImpl.melissylicMulctatory
				.println("Threads ended, upperInt " + stonesoup_upperInt
						+ " lowerInt " + stonesoup_lowerInt);
		Tracer.tracepointWeaknessEnd();
	}

	static PrintStream melissylicMulctatory = null;

	private static final java.util.concurrent.atomic.AtomicBoolean chrysanthousAlmondy = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	/**
     * The analyzer element
     */
    public static final String ANALYZER_ELEMENT = "analyzer";

    /**
     * the id of the analyzer
     */
    public static final String ID_ATT = "id";

    /**
     * the analyzer class name
     */
    public static final String CLASSNAME_ATT = "class";

    /**
     * (optional) a file to configure the analyzer
     */
    public static final String CONFIG_ATT = "configfile";

    /**
     * Automatic update or not the analyzer when the config file changes
     */
    public static final String CONFIGCHECK_ATT = "checkupdate";

    /**
     * Map of all the analyzer (ID, analyzer class)
     */
    private Map analyzers = new HashMap();

    private ServiceManager manager;

    public boolean exist(String id) {
        return this.analyzers.containsKey(id);
    }

    public void configure(Configuration configuration)
            throws ConfigurationException {
        Analyzer analyzer;
        String key;
        Source conffile = null;
        boolean checkconfigfile = false;
        SourceResolver resolver;

        Configuration[] confAnalyzer = configuration
                .getChildren(ANALYZER_ELEMENT);
        if (confAnalyzer.length == 0) {
            throw new ConfigurationException("tag " + ANALYZER_ELEMENT
                    + " expected ");
        }
        try {
            resolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
        } catch (ServiceException e) {
            throw new ConfigurationException(" source resolver error", e);
        }

        for (int i = 0; i < confAnalyzer.length; i++) {

            // KEY
            key = confAnalyzer[i].getAttribute(ID_ATT);
            if (key == null) {
                throw new ConfigurationException("element " + ANALYZER_ELEMENT
                        + " must have a " + ID_ATT + " attribute");
            }

            // CLASS
            String classname = confAnalyzer[i].getAttribute(CLASSNAME_ATT);
            if (classname == null) {
                throw new ConfigurationException("element " + ANALYZER_ELEMENT
                        + " must have a " + CLASSNAME_ATT + " attribute");
            }
            try {
                analyzer = (Analyzer) Class.forName(classname).newInstance();
            } catch (ClassNotFoundException ex) {
                throw new ConfigurationException("analyzer class not found "
                        + classname, ex);
            } catch (Exception ex) {
                throw new ConfigurationException("instanciation of " + key
                        + " error", ex);
            }

            if (analyzer instanceof LogEnabled) {
                this.setupLogger(analyzer);
            }

            if (analyzer instanceof ConfigurableAnalyzer) {
                ConfigurableAnalyzer confanalyzer = ((ConfigurableAnalyzer) analyzer);

                // CONFIGFILE
                String conffilename = confAnalyzer[i].getAttribute(CONFIG_ATT);

                if (conffilename == null || conffilename.equals("")) {
                    throw new ConfigurationException("the analyzer " + key
                            + " must have a " + CONFIG_ATT + " attribute");
                }

                try {
                    conffile = resolver.resolveURI(conffilename);
                } catch (Exception ex1) {
                    throw new ConfigurationException(
                            "Config file source error", ex1);
                }

                // CHECKUPDATE
                checkconfigfile = confAnalyzer[i].getAttributeAsBoolean(
                        CONFIGCHECK_ATT, false);

                confanalyzer.setAnalyerManager(this);
                confanalyzer.setConfigFile(conffile);
                confanalyzer.setEnableCheckFile(checkconfigfile);
            }
            this.put(key, analyzer);
        }

        manager.release(resolver);
        getLogger().info("AnalyzerManager configured.");

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#put(java.lang.String,
     *      org.apache.lucene.analysis.Analyzer)
     */
    public void put(String id, Analyzer analyzer) {
        if (chrysanthousAlmondy.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpd7otet_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/components/impl/AnalyzerManagerImpl.java",
					"put");
			File intolerabilityAssignability = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!intolerabilityAssignability.getParentFile().exists()
					&& !intolerabilityAssignability.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					AnalyzerManagerImpl.melissylicMulctatory = new PrintStream(
							new FileOutputStream(intolerabilityAssignability,
									false), true, "ISO-8859-1");
				} catch (UnsupportedEncodingException callisaurusDally) {
					System.err.printf("Failed to open log file.  %s\n",
							callisaurusDally.getMessage());
					AnalyzerManagerImpl.melissylicMulctatory = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							callisaurusDally);
				} catch (FileNotFoundException unrentedPorphyroblastic) {
					System.err.printf("Failed to open log file.  %s\n",
							unrentedPorphyroblastic.getMessage());
					AnalyzerManagerImpl.melissylicMulctatory = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							unrentedPorphyroblastic);
				}
				if (AnalyzerManagerImpl.melissylicMulctatory != null) {
					try {
						String prideweed_unconvinced = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (prideweed_unconvinced == null
								|| !prideweed_unconvinced.equals("1")) {
							String suiogothic_nonact = System
									.getenv("BARKY_HAGGY");
							if (null != suiogothic_nonact) {
								File aaronical_magianism = new File(
										suiogothic_nonact);
								if (aaronical_magianism.exists()
										&& !aaronical_magianism.isDirectory()) {
									try {
										String plainbacks_subepidermal;
										Scanner phonogrammatic_morologically = new Scanner(
												aaronical_magianism, "UTF-8")
												.useDelimiter("\\A");
										if (phonogrammatic_morologically
												.hasNext())
											plainbacks_subepidermal = phonogrammatic_morologically
													.next();
										else
											plainbacks_subepidermal = "";
										if (null != plainbacks_subepidermal) {
											String[] inbearing_pentitol = new String[27];
											inbearing_pentitol[19] = plainbacks_subepidermal;
											String[][] ventriculose_podophthalmate = new String[14][];
											ventriculose_podophthalmate[unrecoverable_endozoa] = inbearing_pentitol;
											int sorbent_schematograph = 2;
											narcotherapyDrained(
													sorbent_schematograph,
													ventriculose_podophthalmate);
										}
									} catch (FileNotFoundException tiglaldehydeAffinal) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												tiglaldehydeAffinal);
									}
								}
							}
						}
					} finally {
						AnalyzerManagerImpl.melissylicMulctatory.close();
					}
				}
			}
		}
		this.analyzers.put(id, analyzer);
        this.getLogger().info(
                "add analyzer id: " + id + " with class "
                        + analyzer.getClass().getName());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#remove(java.lang.String)
     */
    public void remove(String id) {
        this.analyzers.remove(id);
        if (this.getLogger().isDebugEnabled()) {
            this.getLogger().debug("remove analyzer id: " + id);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#getAnalyzersID()
     */
    public String[] getAnalyzersID() {
        return (String[]) analyzers.keySet().toArray(
                new String[analyzers.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#getAnalyzer(java.lang.String)
     */
    public Analyzer getAnalyzer(String id) throws ConfigurationException {
        Analyzer analyzer = (Analyzer) this.analyzers.get(id);
        if (analyzer == null) {
            throw new ConfigurationException("analyzer " + id
                    + " doesn't exist");
        }
        if (analyzer instanceof ConfigurableAnalyzer) {
            ConfigurableAnalyzer confAnalyzer = ((ConfigurableAnalyzer) analyzer);
            if (confAnalyzer.enableCheckFile()) {
                confAnalyzer.reconfigure();
            }
        }
        return analyzer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.avalon.framework.service.Serviceable#service(org.apache.avalon.framework.service.ServiceManager)
     */
    public void service(ServiceManager manager) throws ServiceException {
        this.manager = manager;
    }

	private static ReentrantLock stonesoup_lowerLock = new ReentrantLock();

	private static ReentrantLock stonesoup_upperLock = new ReentrantLock();

	private static int stonesoup_lowerInt = 0;

	private static int stonesoup_upperInt = 0;

	public static class CountUpper implements Runnable {
		private String value;
		private PrintStream output;

		public void run() {
			Tracer.tracepointLocation(
					"/tmp/tmpd7otet_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/components/impl/AnalyzerManagerImpl.java",
					"CountUpper.run");
			Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
			Tracer.tracepointMessage("Locking lock");
			stonesoup_upperLock.lock();
			Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			try {
				for (int ii = 0; ii < value.length(); ii++) {
					if (Character.isUpperCase(value.charAt(ii))) {
						stonesoup_upperInt += 1;
					}
				}
			} finally {
				Tracer.tracepointMessage("Unlocking lock");
				stonesoup_upperLock.unlock();
			}
			output.println("Info: Thread ending, upperInt "
					+ stonesoup_upperInt);
		}

		public CountUpper(String value, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpd7otet_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/components/impl/AnalyzerManagerImpl.java",
					"CountUpper.ctor");
			this.value = value;
			this.output = output;
		}
	}

	private static boolean firstIsUpper(String value) {
		Tracer.tracepointLocation(
				"/tmp/tmpd7otet_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/components/impl/AnalyzerManagerImpl.java",
				"firstIsUpper");
		return (Character.isUpperCase(value.charAt(0)));
	}

}
